--!parent
--!encoding=utf-8

--****************************************************************************
-- @file      LSW CORE LIB: CURL
-- @author    Valentin Schmidt
-- @version   0.2
-- @requires  xtra("Curl")
--****************************************************************************

property $

property _DetachClass

----------------------------------------
-- @constructor
----------------------------------------
on new (me)
	
  $.loadXtras(me.libdir, ["Curl.x32"], ["CurlXtra.xtra"])
  
  $.import("detacher")
    
  -- LOAD EXTENSIONS
  $.inherit(me, $.include(me.libdir&"curlopt.ls"))
  $.inherit(me, $.include(me.libdir&"curlinfo.ls"))
  $.inherit(me, $.include(me.libdir&"curlform.ls"))

  -- LOAD "DETACH" HELPER CLASS
  me._DetachClass = $.include(me.libdir&"detach.ls")

  return me
end

----------------------------------------
-- FACTORY METHOD
-- @return {instance} Curl handle (xtra instance)
----------------------------------------
on init (me)
  return xtra("Curl").new()
end

----------------------------------------
-- Used for "detaching" xtra callbacks from original thread,
-- so scripting errors in the callback's code don't fail silently
-- @param {instance} ch - Curl handle
-- @param {symbol} cbHandler - Callback handler
-- @param {object} [cbTarget=_movie] - Callback target
-- @param {bool} [passImmediately=false] - If specified, received data chunks are passed to callback immediately
-- @return {integer} Curl multi error code (0 = no error)
----------------------------------------
on execAsyncDetached (me, ch, cbHandler, cbTarget, passImmediately)
  if voidP(cbTarget) then cbTarget = _movie
  obj = me._DetachClass.new(cbHandler, cbTarget, ch)
  return ch.execAsync(#_detach, obj, passImmediately)
end

----------------------------------------
-- Generates URL-encoded query string
-- @param {propList} param_obj
-- @return {string}
----------------------------------------
on httpBuildQuery (me, param_obj)
  query_str = ""
  cnt = count(param_obj)
  repeat with i = 1 to cnt
    put param_obj.getPropAt(i)&"="&curl_escape(param_obj[i])&"&" after query_str
  end repeat
  delete the last char of query_str
  return query_str
end

----------------------------------------
-- Parse HTTP result headers
-- @param {string} headerStr
-- @return {propList}
----------------------------------------
on parseHeaders (me, resultStr)  
  headerSep = numtochar(13)&numtochar(10)&numtochar(13)&numtochar(10)
  lineSep = numtochar(13)&numtochar(10)
  parts = me._explode(headerSep, resultStr)
  headerStr = parts[1]
  lines = me._explode(lineSep, headerStr)
  headers = [:]
  repeat with s in lines
    if not (s contains ":") then next repeat
    parts = me._explode(":", s)
    k = parts[1]
    v = parts[2]
    if v starts " " then delete char 1 of v
    headers[k] = v
  end repeat
  return headers
end

----------------------------------------
-- Explode string into list
-- @param {string} delim
-- @param {string} str
-- @return {list}
----------------------------------------
on _explode (me, delim, str)
  l = []
  if voidP(str) then return l
  dl = delim.length
  repeat while true
    pos = offset(delim,str)
    if pos=0 then exit repeat
    s = str.char[1..pos-1]
    l.add(s)
    delete char 1 to pos+dl-1 of str
  end repeat
  if pos=0 then pos=1-dl
  l.add(str.char[pos+dl..str.length])
  return l
end
